use num::Integer;

/**
 * Discrete/Modular exponentiation
 *
 * Highly memory efficient because the full result is never stored, but shortened by defined modulo instead.
 * We can use that because the prime divisor required for our algorithm is guarenteed to be smaller
 * than n.
 *
 * Counterpart function to the discrete logarithm.
 */
pub fn mod_pow<T>(mut base: T, mut exp: T, r#mod: T) -> T
where
    T: Integer + Copy + std::ops::DivAssign + From<u8>,
    u64: From<T>,
    u128: From<T>,
{
    let mut result: T = T::one();
    base = base % r#mod;
    while exp > T::zero() {
        if exp.is_odd() {
            result = (result * base) % r#mod;
        }
        base = (base * base) % r#mod;
        exp /= T::from(2);
    }
    result
}